﻿// DopStack
// (c) 2020 Steve Banham

scriptName = "DopStack";
scriptVersion = "1.5.3";
scriptDate = "12/7/2020";
scriptCopyright = "©2020 Steve Banham";
scriptMinVersion = "12.21";
scriptDesc = "Drop files and folders onto DopStack from anywhere to add to the stack, then paste or drag the stack anywhere.";
scriptUrl = "https://resource.dopus.com/t/dopstack/35671";

var LAST_DROP_EXISTS = false;
var COMPRESS_EMAIL_PATHS = DOpus.NewVector();
var NAME_USE = "";
var IMG_STACK, IMG_TARGET;

var BACKGROUND_COLOUR, CLIPBOARD_NOMATCH, TEXT_COLOUR, FILES_BG_COLOUR, FILES_FG_COLOUR, FOLDERS_BG_COLOUR, FOLDERS_FG_COLOUR, VIEWER_COLOUR;
var SAVE_LOAD_PATH, TEXT_EDITOR_PATH;

function OnInit(initData) {
    initData.name = scriptName;
    initData.version = scriptVersion;
    initData.copyright = scriptCopyright;
    initData.desc = scriptDesc;
    initData.default_enable = true;
    initData.min_version = scriptMinVersion;
    initData.url = "https://resource.dopus.com/t/dopstack/35671";

    initData.config_desc = DOpus.Create.Map();
    initData.config_groups = DOpus.Create.Map();
    var configName = "";
    
    configName = "background_colour";
    initData.Config[configName] = "#caced5";
    initData.config_desc(configName) = "The colour used as a background for the dialog (Use hexidecimal).";
    initData.config_groups(configName) = "DopStack Theme";

    configName = "text_colour";
    initData.Config[configName] = "#000000";
    initData.config_desc(configName) = "The colour used to display text in the dialog (Use hexidecimal).";
    initData.config_groups(configName) = "DopStack Theme";
    
    configName = "stack_stats_bold";
    initData.Config[configName] = false;
    initData.config_desc(configName) = "Use a bold font for the stack statistics.";
    initData.config_groups(configName) = "DopStack Theme";

    configName = "stack_label_bold";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "Use a bold font for the stack label.";
    initData.config_groups(configName) = "DopStack Theme";

    configName = "clipboard_nomatch";
    initData.Config[configName] = "#ff9f40";
    initData.config_desc(configName) = "Label highlight colour when clipboard does not match the stack.";
    initData.config_groups(configName) = "DopStack Theme";

    configName = "stay_on_top";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "Keep the DopStack dialog on top of all windows.";
    initData.config_groups(configName) = "DopStack Options";

    configName = "stack_label_text";
    initData.Config[configName] = "[default stack]";
    initData.config_desc(configName) = "Default text used to identify the stack.";
    initData.config_groups(configName) = "DopStack Options";

    configName = "confirm_exit";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "Prompts for confirmation before exiting if the Stack contains items."
    initData.config_groups(configName) = "DopStack Options";

    configName = "default_save_load_path";
    initData.Config[configName] = "/desktop";
    initData.config_desc(configName) = "Default path used to save or load a .stack file."
    initData.config_groups(configName) = "DopStack Options";

    configName = "remember_position";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "The dialog remembers it's last size and position.";
    initData.config_groups(configName) = "DopStack Options";

    configName = "extended_viewer_info";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "Show extended information (size, date, etc).";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "group_viewer_list";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "Group files and folders.";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "icon_mode";
    initData.Config[configName] = false;
    initData.config_desc(configName) = "Open Stack Viewer in icon mode.";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "remember_position_viewer";
    initData.Config[configName] = true;
    initData.config_desc(configName) = "The Stack Viewer remembers it's last size and position.";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "text_editor_label";
    initData.Config[configName] = "Notepad...";
    initData.config_desc(configName) = "The label for the text editor button.";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "text_editor_path";
    initData.Config[configName] = "/system\\notepad.exe";
    initData.config_desc(configName) = "The path to your preferred text editor.";
    initData.config_groups(configName) = "Stack Viewer Options";

    configName = "files_fg_colour";
    initData.Config[configName] = "#000000";
    initData.config_desc(configName) = "Text foreground colour for files.";
    initData.config_groups(configName) = "Stack Viewer Theme";

    configName = "files_bg_colour";
    initData.Config[configName] = "#FFFFFF";
    initData.config_desc(configName) = "Text background colour for files.";
    initData.config_groups(configName) = "Stack Viewer Theme";

    configName = "folders_fg_colour";
    initData.Config[configName] = "#0000a8";
    initData.config_desc(configName) = "Text foreground colour for folders.";
    initData.config_groups(configName) = "Stack Viewer Theme";

    configName = "folders_bg_colour";
    initData.Config[configName] = "#FFFFFF";
    initData.config_desc(configName) = "Text background colour for folders.";
    initData.config_groups(configName) = "Stack Viewer Theme";

    configName = "viewer_colour";
    initData.Config[configName] = "#FFFFFF";
    initData.config_desc(configName) = "Background colour for the Stack Viewer list.";
    initData.config_groups(configName) = "Stack Viewer Theme";

    var vecArc = DOpus.NewVector();
    vecArc.push_back(0);
    vecArc.push_back("7-Zip");
    vecArc.push_back("Zip");
    vecArc.push_back("Rar");
    
	configName = "archive_format";
	initData.config[configName] = vecArc;
    initData.config_desc(configName) = "The archive format used to compress the stack.";
    initData.config_groups(configName) = "DopStack Options";

    var vecDblClk = DOpus.NewVector();
	vecDblClk.push_back(0);
	vecDblClk.push_back("Show with DOpus viewer");
	vecDblClk.push_back("Default Open action");
    vecDblClk.push_back("View Properties");

    configName = "dblclk_viewer";
	initData.config[configName] = vecDblClk;
    initData.config_desc(configName) = "The action which occurs when an item is double clicked.";
    initData.config_groups(configName) = "Stack Viewer Options";

    var vecTargetStyle = DOpus.NewVector();
	vecTargetStyle.push_back(0);
    vecTargetStyle.push_back("Scope");
    vecTargetStyle.push_back("Cat");

	configName = "target_style";
	initData.config[configName] = vecTargetStyle;
    initData.config_desc(configName) = "The style for the target icon (empty stack).";
    initData.config_groups(configName) = "DopStack Theme";

    var vecCross = DOpus.NewVector();
	vecCross.push_back(0);
    vecCross.push_back("Grey");
    vecCross.push_back("Light Grey");
    vecCross.push_back("Red");
    vecCross.push_back("Yellow");
    vecCross.push_back("Blue");
	vecCross.push_back("White");
    vecCross.push_back("Black");

	configName = "target_colour";
	initData.config[configName] = vecCross;
    initData.config_desc(configName) = "The colour for the target icon (empty stack).";
    initData.config_groups(configName) = "DopStack Theme";

    var vecFile = DOpus.NewVector();
	vecFile.push_back(0);
    vecFile.push_back("Grey");
    vecFile.push_back("Light Grey");
    vecFile.push_back("Red");
    vecFile.push_back("Yellow");
    vecFile.push_back("Blue");
	vecFile.push_back("White");
    vecFile.push_back("Black");

	configName = "stack_colour";
	initData.config[configName] = vecFile;
    initData.config_desc(configName) = "The colour for the stack icon (when the stack has items).";
    initData.config_groups(configName) = "DopStack Theme";

    var cmd = initData.AddCommand();
    cmd.name = "DopStack";
    cmd.method = "onDopStack";
    cmd.desc = scriptDesc;
    cmd.label = "DopStack";
    cmd.template = "ADDTOSTACK/O[<default>,allsource,alldestination,selecteddestination],STACKLABEL/K";
}

function onDopStack(scriptCmdData) {

    BACKGROUND_COLOUR = Script.Config["background_colour"];
        if (BACKGROUND_COLOUR.search("#") != 0) BACKGROUND_COLOUR = "#" + BACKGROUND_COLOUR;
    CLIPBOARD_NOMATCH = Script.Config["clipboard_nomatch"];
        if (CLIPBOARD_NOMATCH.search("#") != 0) CLIPBOARD_NOMATCH = "#" + CLIPBOARD_NOMATCH;
    TEXT_COLOUR = Script.Config["text_colour"];
        if (TEXT_COLOUR.search("#") != 0) TEXT_COLOUR = "#" + TEXT_COLOUR;
    FILES_BG_COLOUR = Script.Config["files_bg_colour"];
        if (FILES_BG_COLOUR.search("#") != 0) FILES_BG_COLOUR = "#" + FILES_BG_COLOUR;
    FILES_FG_COLOUR = Script.Config["files_fg_colour"];
        if (FILES_FG_COLOUR.search("#") != 0) FILES_FG_COLOUR = "#" + FILES_FG_COLOUR;
    FOLDERS_BG_COLOUR = Script.Config["folders_bg_colour"];
        if (FOLDERS_BG_COLOUR.search("#") != 0) FOLDERS_BG_COLOUR = "#" + FOLDERS_BG_COLOUR;
    FOLDERS_FG_COLOUR = Script.Config["folders_fg_colour"];
        if (FOLDERS_FG_COLOUR.search("#") != 0) FOLDERS_FG_COLOUR = "#" + FOLDERS_FG_COLOUR;
    VIEWER_COLOUR = Script.Config["viewer_colour"];
        if (VIEWER_COLOUR.search("#") != 0) VIEWER_COLOUR = "#" + VIEWER_COLOUR;

    SAVE_LOAD_PATH = DOpus.FSUtil.Resolve(Script.Config["default_save_load_path"]);
    TEXT_EDITOR_PATH = DOpus.FSUtil.Resolve(Script.Config["text_editor_path"]);

    var vecStack = DOpus.NewVector();
    var vecLastDrop = DOpus.NewVector();
    var vecBeforeDrop = DOpus.NewVector();
    var vecBeforeDrag = DOpus.NewVector();
    var vecError = DOpus.NewVector();

    var srcTab = scriptCmdData.func.sourcetab;
    var desTab = scriptCmdData.func.desttab;

    var dlg = DOpus.Dlg;
    dlg.title = scriptName;
    dlg.template = "dlgDopStack";

    Script.Config["remember_position"] ? dlg.LoadPosition("Steve_DopStack") : null;
    Script.Config["stay_on_top"] ? dlg.top = true : null;

    dlg.detach = true;
    dlg.want_close = true;

    dlg.Create();

    checkConfig(dlg, vecError);

    switch (Script.Config["target_style"]) {
        case 0: var imgStyle = "scope";
            break;
        case 1: var imgStyle = "cat";
    }

    switch (Script.Config["target_colour"]) {
        case 0: var imgTarColour = "_grey.png";
            break;
        case 1: var imgTarColour = "_lightgrey.png";
            break;
        case 2: var imgTarColour = "_red.png";
            break;
        case 3: var imgTarColour = "_yellow.png";
            break;
        case 4: var imgTarColour = "_blue.png";
            break;
        case 5: var imgTarColour = "_white.png";
            break;
        case 6: var imgTarColour = "_black.png";
    }

    switch (Script.Config["stack_colour"]) {
        case 0: var imgStaColour = "stack_grey.png";
            break;
        case 1: var imgStaColour = "stack_lightgrey.png";
            break;
        case 2: var imgStaColour = "stack_red.png";
            break;
        case 3: var imgStaColour = "stack_yellow.png";
            break;
        case 4: var imgStaColour = "stack_blue.png";
            break;
        case 5: var imgStaColour = "stack_white.png";
            break;
        case 6: var imgStaColour = "stack_black.png";
    }

    IMG_TARGET = Script.LoadImage(imgStyle + imgTarColour);
    IMG_STACK = Script.LoadImage(imgStaColour);

    dlg.Control("staticDrop").label = IMG_TARGET;

    dlg.Control("staticDrop").bg = BACKGROUND_COLOUR;
    dlg.Control("staticStats").bg = BACKGROUND_COLOUR;
    dlg.Control("staticStats").fg = TEXT_COLOUR;
    dlg.Control("staticLabel").bg = BACKGROUND_COLOUR;
    dlg.Control("staticLabel").fg = TEXT_COLOUR;
    dlg.Control("staticName").bg = BACKGROUND_COLOUR;
    dlg.Control("staticName").fg = TEXT_COLOUR;

    dlg.Control("staticError").fg = "#000000";
    dlg.Control("staticError").bg = "#ffff00";

    dlg.Control("staticName").visible = false;
    dlg.Control("editName").visible = false;
    dlg.Control("btnNo").visible = false;
    dlg.Control("staticName").enabled = false;
    dlg.Control("editName").enabled = false;
    dlg.Control("btnNo").enabled = false;
    
    dlg.Control("staticLabel").label = Script.Config["stack_label_text"];

    var args = scriptCmdData.func.args;
    args.got_arg.stacklabel ? dlg.Control("staticLabel").label = args.stacklabel : null;
 
    Script.Config["stack_label_bold"] ? dlg.Control("staticLabel").style = "b" : null;
    Script.Config["stack_stats_bold"] ? dlg.Control("staticStats").style = "b" : null;

    if (args.got_arg.addtostack) {
        switch (args.addtostack) {
            case "allsource":
                if (srcTab.selstats > 0) {
                    vecStack.clear();
                    for (var eItems = new Enumerator(srcTab.all); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        vecStack.push_back = thisItem.realpath;
                    }
                    DOpus.SetClip(vecStack);
                }
                break;
            case "alldestination":
                if (desTab.selstats > 0) {
                    vecStack.clear();
                    for (var eItems = new Enumerator(desTab.all); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        vecStack.push_back = thisItem.realpath;
                    }
                    DOpus.SetClip(vecStack);
                }
                break;
            case "selecteddestination":
                if (desTab.selstats > 0) {
                    vecStack.clear();
                    for (var eItems = new Enumerator(desTab.selected); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        vecStack.push_back = thisItem.realpath;
                    }
                    DOpus.SetClip(vecStack);
                }
                break;
            default: 
                if (srcTab.selstats > 0) {
                    vecStack.clear();
                    for (var eItems = new Enumerator(srcTab.selected); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        vecStack.push_back = thisItem.realpath;
                    }
                    DOpus.SetClip(vecStack);
                }
        }
    }
    updateDialog(dlg, vecStack);

    dlg.Show();
    dlg.SetTimer(500, "timer");

    while (true) {
        var msg = dlg.GetMsg();
        if (!msg.result) break;

        if (msg.event == "close") {
            exitConfirm(dlg, vecStack);
        }

        if (msg.event == "drag") {
            if (vecStack.count > 0) {
                vecBeforeDrag.assign(vecStack);
                dlg.Drag(vecStack);
                vecStack.assign(vecBeforeDrag);
                updateDialog(dlg, vecStack);
            }
        }

        if (msg.event == "click" && msg.control == "btnEnter") {
            updateDialog(dlg, vecStack);
            if (dlg.Control("editName").value != "") { 
                if (NAME_USE == "label") {
                    dlg.Control("staticLabel").label = dlg.Control("editName").value;
                }
                if (NAME_USE == "email") {
                    archiveEMailCreate(dlg, vecStack, scriptCmdData);
                }
            }
        }

        if (msg.event == "click" && msg.control == "btnNo") {
            updateDialog(dlg, vecStack);
        }

        if (msg.event == "timer") {
            dlg.KillTimer("timer");
            if (vecStack.count > 0) {
                var vecClip = DOpus.GetClip();
                if (DOpus.GetClipFormat() == "files") {
                    var factory = DOpus.Create();
                    var strClip = factory.StringSet();
                    var strStack = factory.StringSet();
                    
                    for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        strClip.insert(thisItem.realpath);
                    }

                    for (var eItems = new Enumerator(vecClip); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        strClip.erase(thisItem.realpath);
                    }

                    if (strClip.count == 0) {
                        dlg.Control("staticLabel").bg = BACKGROUND_COLOUR;
                    }
                    else {
                        dlg.Control("staticLabel").bg = CLIPBOARD_NOMATCH;
                    }
                }
                else {
                    dlg.Control("staticLabel").bg = CLIPBOARD_NOMATCH;
                }
            }
            else {
                dlg.Control("staticLabel").bg = BACKGROUND_COLOUR;
            }

            checkExists(vecStack);
            if (dlg.Control("editName").visible == false) {
                updateDialog(dlg, vecStack); 
            }
            dlg.SetTimer(500, "timer");
        }

        if (msg.event == "drop") {
                dlg.KillTimer("timer");
                LAST_DROP_EXISTS = true;
                vecBeforeDrop.assign(vecStack);
                vecLastDrop.clear();
                for (var eItems = new Enumerator(msg.object); !eItems.atEnd(); eItems.moveNext()) {
                    var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                    vecStack.push_back = thisItem.realpath;
                    vecLastDrop.push_back = thisItem.realpath;
                    DOpus.SetClip(vecStack);                 
                }
                
                stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
                updateDialog(dlg, vecStack);
                dlg.SetTimer(500, "timer");
        }

        if (msg.event == "rclick") {
            dlg.KillTimer("timer");
            popupMenu(dlg, vecStack, vecLastDrop, vecBeforeDrop, scriptCmdData, srcTab);
            dlg.SetTimer(500, "timer");
        }

        if (msg.event == "dblclk") {   
            DOpus.SetClip(vecStack);
            dlg.Control("staticStats").label = "Copied";
            DOpus.Delay(1000);
            updateDialog(dlg, vecStack);
        }

        if (msg.event == "click" && msg.control == "staticError") {
            displayConfigErrors(dlg, vecError);
        }
    }
    
    Script.Config["remember_position"] ? dlg.SavePosition("Steve_DopStack") : null;

    var fso = new ActiveXObject("Scripting.FilesystemObject");
    var tmpFolder = fso.GetSpecialFolder(2);
    var cmd = scriptCmdData.func.command;
    cmd.RunCommand("Delete NORECYCLE QUIET FILE=\"" + tmpFolder + "\\Stack.txt" + "\"");
    for (var eItems = new Enumerator(COMPRESS_EMAIL_PATHS); !eItems.atEnd(); eItems.moveNext()) {
        cmd.RunCommand("Delete QUIET NORECYCLE \"" + eItems.item() + "\"");
    }
}

function displayConfigErrors(dlg, vecError) {
    var dlgE = DOpus.Dlg;
    dlgE.template = "dlgConfigErrors";
    dlgE.window = dlg;
    dlgE.disable_window = dlg;
    dlgE.LoadPosition("Steve_DopStack_ConfigErrors");
    dlgE.detach = true;
    dlgE.Create();
    dlgE.Control("listErrors").columns.AddColumn("Name");
    dlgE.Control("listErrors").columns.AddColumn("Value");
    for (i=0; i < vecError.count; i ++) {
        var a = dlgE.Control("listErrors").AddItem(vecError[i]);
        dlgE.Control("listErrors").GetItemAt(a).subitems(0) = Script.Config[vecError[i]];
    }
    dlgE.Control("listErrors").columns.AutoSize();
    dlgE.Show();
    while (true) {
        var msg = dlgE.GetMsg();
        if (!msg.result) break;
    }
    dlgE.SavePosition("Steve_DopStack_ConfigErrors");
    return;
}

function checkExists(vecStack) {
    var vecTemp = DOpus.NewVector();
    for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
        if (DOpus.FSUtil.Exists(thisItem)) {
            vecTemp.push_back = thisItem.realpath;
        }
    }
    vecStack.assign(vecTemp);   
}

function checkConfig(dlg, vecError) {

    if (!hexCheck(BACKGROUND_COLOUR)) {
        BACKGROUND_COLOUR = "#caced5";
        vecError.push_back = "background_colour";
    }
    if (!hexCheck(CLIPBOARD_NOMATCH)) {
        CLIPBOARD_NOMATCH = "#ff9f40";
        vecError.push_back = "clipboard_nomatch";
    }
    if (!hexCheck(TEXT_COLOUR)) {
        TEXT_COLOUR = "#000000";
        vecError.push_back = "text_colour";
    }
    if (!hexCheck(FILES_BG_COLOUR)) {
        FILES_BG_COLOUR = "#FFFFFF";
        vecError.push_back = "files_bg_colour";
    }
    if (!hexCheck(FILES_FG_COLOUR)) {
        FILES_FG_COLOUR = "#000000";
        vecError.push_back = "files_fg_colour";
    }
    if (!hexCheck(FOLDERS_BG_COLOUR)) {
        FOLDERS_BG_COLOUR = "#FFFFFF";
        vecError.push_back = "folders_bg_colour";
    }
    if (!hexCheck(FOLDERS_FG_COLOUR)) {
        FOLDERS_FG_COLOUR = "#0000a8";
        vecError.push_back = "folders_fg_colour";
    }
    if (!hexCheck(VIEWER_COLOUR)) {
        VIEWER_COLOUR = "#FFFFFF";
        vecError.push_back = "viewer_colour";
    }

    if (!DOpus.FSUtil.Exists(SAVE_LOAD_PATH)) {
        SAVE_LOAD_PATH = DOpus.FSUtil.Resolve("/desktop");
        vecError.push_back = "default_save_load_path";
    }

    if (!DOpus.FSUtil.Exists(TEXT_EDITOR_PATH)) {
        TEXT_EDITOR_PATH = DOpus.FSUtil.Resolve("/system\\notepad.exe");
        vecError.push_back = "text_editor_path";
    }

    if (vecError.count > 0) {
        dlg.Control("staticError").visible = true;
        dlg.Control("staticError").enabled = true;
    }
}

function hexCheck(hexColour) {
    var reg = /[0-9A-Fa-f]{6}/g;
    if (hexColour.search("#") == 0) {
        hexColour = hexColour.substring(1);
        if (reg.test(hexColour)) {
            return true;
        }
    }
    else return false;
}

function exitConfirm(dlg, vecStack) {
    if (Script.Config["confirm_exit"] == true && vecStack.count > 0) {
        
        switch (vecStack.count) {
            case 1: var strCnt = "is 1 item";
                break;
            default: var strCnt = "are " + vecStack.count + " items";
        }
        var strMsg = "There " + strCnt + " in the stack\n\nDo you want to save the Stack before exiting?";

        dlgCon = DOpus.Dlg;
        dlgCon.window = dlg;
        dlgCon.disable_window = dlg;
        dlgCon.title = "Confirm";
        dlgCon.message = strMsg;
        dlgCon.buttons = "Save|Don't Save|Cancel";
        dlgCon.icon = "warning";

        var result = dlgCon.Show();
        if (result == 1) {
            saveStack(dlg, vecStack);
            dlg.EndDlg();
        } 
        if (result == 2) {
            dlg.EndDlg();
        }
    }
    else {
        dlg.EndDlg();
    }
}

function archiveEMailCreate(dlg, vecStack, scriptCmdData) {
    var fso = new ActiveXObject("Scripting.FilesystemObject");
    var tmpFolder = fso.GetSpecialFolder(2);
    var cmd = scriptCmdData.func.command;
    cmd.SetDest(String(tmpFolder));
    cmd.SetFiles(vecStack);
    
    if (DOpus.FSUtil.Exists(tmpFolder + "\\" + dlg.Control("editName").value)) {
        cmd.RunCommand("Delete QUIET NORECYCLE \"" + tmpFolder + "\\" + dlg.Control("editName").value + "\"");
    }

    switch (Script.Config["archive_format"]) {
        case 0: // .7z
            cmd.RunCommand("COPY ARCHIVE=.7z CREATEFOLDER=\"" + dlg.Control("editName").value + "\"");
            break;
        case 1: // .zip
            cmd.RunCommand("COPY ARCHIVE CREATEFOLDER=\"" + dlg.Control("editName").value + "\"");
            break;
        case 2: // .rar
            cmd.RunCommand("COPY ARCHIVE=.rar CREATEFOLDER=\"" + dlg.Control("editName").value + "\"");
    }
    cmd.RunCommand("Copy SENDMAIL FILE=\"" + tmpFolder + "\\" + dlg.Control("editName").value + "\"");
    COMPRESS_EMAIL_PATHS.push_back = tmpFolder + "\\" + dlg.Control("editName").value;
}

function archiveCreate(dlg, srcTab, vecStack, scriptCmdData, arcItem) {
    var cmd = scriptCmdData.func.command;
    cmd.SetDest(arcItem.pathpart);
    cmd.SetFiles(vecStack);
    switch (Script.Config["archive_format"]) {
        case 0: // .7z
            cmd.RunCommand("COPY ARCHIVE=.7z CREATEFOLDER=\"" + arcItem.filepart + "\"");
            break;
        case 1: // .zip
            cmd.RunCommand("COPY ARCHIVE CREATEFOLDER=\"" + arcItem.filepart + "\"");
            break;
        case 2: // .rar
            cmd.RunCommand("COPY ARCHIVE=.rar CREATEFOLDER=\"" + arcItem.filepart + "\"");
    }
}

function viewStack(dlg, vecStack, vecBeforeDrop, vecLastDrop, scriptCmdData) {
    
    var map = DOpus.Create.Map();

    var dlgStack = DOpus.Dlg;
    dlgStack.title = "Stack Viewer";
    dlgStack.template = "dlgStackView";
    dlgStack.window = dlg;
    dlgStack.disable_window = dlg;

    Script.Config["remember_position_viewer"] ? dlgStack.LoadPosition("Steve_DopStack_ViewStack") : null;

    dlgStack.detach = true;
    dlgStack.Create();

    dlgStack.Control("listStack").bg = VIEWER_COLOUR;
    dlgStack.Control("listStack").textbg = VIEWER_COLOUR;

    if (Script.Config["icon_mode"] == false) {
        dlgStack.Control("listStack").mode = "details";
        dlgStack.Control("chkDetails").value = true;
    }
    else {
        dlgStack.Control("listStack").mode = "icon";
        dlgStack.Control("chkDetails").value = false;
    }
    
    if (Script.Config["extended_viewer_info"] == true) {
        dlgStack.Control("listStack").columns.AddColumn("Name");
        dlgStack.Control("listStack").columns.AddColumn("Size");
        dlgStack.Control("listStack").columns.AddColumn("Modified");
        dlgStack.Control("listStack").columns.AddColumn("Attr");
        dlgStack.Control("listStack").columns.AddColumn("Location");
    }
    else {
        dlgStack.Control("listStack").columns.AddColumn("Item");
    }

    if (Script.Config["group_viewer_list"] == true) {
        dlgStack.Control("chkGroup").value = true;
    }

    dlgStack.Control("listStack").AddGroup("Folders", 1, "c");
    dlgStack.Control("listStack").AddGroup("Files", 2, "c");
    
    dlgStack.Control("btnNotepad").label = Script.Config["text_editor_label"];
    dlgStack.Control("btnNotepad").focus = true;

    updateViewer(dlg, dlgStack, vecStack, map);

    dlgStack.Show();
    
    while (true) {
        var msg = dlgStack.GetMsg();
        if (!msg.result) break;

        if (msg.event == "drag" && msg.control == "listStack") {
            var vecFile = DOpus.NewVector();
            vecFile.clear();

            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                var thisItem = DOpus.FSUtil.GetItem(strFullPath);
                vecFile.push_back = thisItem.realpath;
                dlg.Drag(vecFile);
            }
        }

        if (msg.event == "drop" && msg.control == "listStack") {
            LAST_DROP_EXISTS = true;
            vecBeforeDrop.assign(vecStack);
            vecLastDrop.clear();
            for (var eItems = new Enumerator(msg.object); !eItems.atEnd(); eItems.moveNext()) {
                var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                vecStack.push_back = thisItem.realpath;
                vecLastDrop.push_back = thisItem.realpath;
                DOpus.SetClip(vecStack);                 
            }            
            stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
            updateDialog(dlg, vecStack);
            updateViewer(dlg, dlgStack, vecStack, map);
        }
        
        if (msg.event == "editchange" && msg.control == "editFilter") {
            updateViewer(dlg, dlgStack, vecStack, map);
        }

        if (msg.event == "click" && dlgStack.Control("chkGroup").focus == true) {
            updateViewer(dlg, dlgStack, vecStack, map);
        }

        if (msg.event == "click" && dlgStack.Control("chkDetails").focus == true) {
            if (dlgStack.Control("chkDetails").value == true) {
                dlgStack.Control("listStack").mode = "details";
            }
            else {
                dlgStack.Control("listStack").mode = "icon";
            }
            updateViewer(dlg, dlgStack, vecStack, map);
        }

        if (msg.event == "click") {
            switch (msg.control) {
                case "btnClear":
                    dlgStack.Control("editFilter").value = "";
                    break;
                case "btnNotepad":
                    var fso = new ActiveXObject("Scripting.FilesystemObject");
                    var tmpFolder = fso.GetSpecialFolder(2);
                    var stackText = fso.OpenTextFile(tmpFolder + "\\Stack.txt", 2, true);
            
                    for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                        var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                        stackText.WriteLine(thisItem.realpath);
                    }
                    stackText.Close();
                    var cmd = scriptCmdData.func.command;
                    cmd.RunCommand("\"" + TEXT_EDITOR_PATH + "\" \"" + tmpFolder + "\\Stack.txt\"");
            }
        }

        if (msg.event == "dblclk" && msg.control == "listStack") {
            var cmd = scriptCmdData.func.command;
            switch (Script.Config["dblclk_viewer"]) {
                case 0: 
                    if (dlgStack.Control("listStack").mode == "details") {
                        if (Script.Config["extended_viewer_info"]) {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                        else {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                    }
                    else {
                        for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                            var key = eItem.item();
                            if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                                strFullPath = map(key);
                            }
                        }
                    }
                    if (DOpus.FSUtil.Exists(strFullPath)) {
                        cmd.RunCommand("Show \"" + strFullPath + "\"");
                    }
                    break;
                case 1:
                    if (dlgStack.Control("listStack").mode == "details") {
                        if (Script.Config["extended_viewer_info"]) {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                        else {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                    }
                    else {
                        for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                            var key = eItem.item();
                            if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                                strFullPath = map(key);
                            }
                        }
                    }
                    if (DOpus.FSUtil.Exists(strFullPath)) {
                        cmd.RunCommand("FileType ACTION=dblclk FILE \"" + strFullPath + "\"");
                    }
                    break;
                case 2:
                    if (dlgStack.Control("listStack").mode == "details") {
                        if (Script.Config["extended_viewer_info"]) {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                        else {
                           var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                        }
                    }
                    else {
                        for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                            var key = eItem.item();
                            if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                                strFullPath = map(key);
                            }
                        }
                    }
                    if (DOpus.FSUtil.Exists(strFullPath)) {
                        cmd.RunCommand("Properties FILE \"" + strFullPath + "\"");
                    }
            }
        }

        if (msg.event == "rclick" && msg.Control == "listStack") {
            if (dlgStack.Control("listStack").value > -1) {
                viewPopupMenu(dlg, dlgStack, vecStack, scriptCmdData, map);
            }
        }
    }
    Script.Config["remember_position_viewer"] ? dlgStack.SavePosition("Steve_DopStack_ViewStack") : null;
}

function updateViewer(dlg, dlgStack, vecStack, map) {

    map.Clear();

    dlgStack.Control("listStack").RemoveItem(-1);
    dlgStack.Control("chkGroup").value ? dlgStack.Control("listStack").EnableGroupView(true) : dlgStack.Control("listStack").EnableGroupView(false);

    if (dlgStack.Control("editFilter").value != "") {
        for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
            var thisItem = DOpus.FSUtil.GetItem(eItems.item());
            var strTemp = String(thisItem.realpath);
            strTemp = strTemp.toLowerCase();
            var strFilter = dlgStack.Control("editFilter").value;
            strFilter = strFilter.toLowerCase();
            if (strTemp.search(dlgStack.Control("editFilter").value) > -1) {
                if (dlgStack.Control("chkDetails").value == true) {
                    populateListDetails(dlgStack, thisItem);
                }
                else {
                    populateListIcon(dlgStack, thisItem);
                }
            }
        }
    }
    else {
        for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
            var thisItem = DOpus.FSUtil.GetItem(eItems.item());
            if (dlgStack.Control("chkDetails").value == true) {
                populateListDetails(dlgStack, thisItem);
            }
            else {
                populateListIcon(dlgStack, thisItem, map);
            }
        }
    }
    
    dlgStack.Control("listStack").columns.AutoSize();
}

function populateListDetails(dlgStack, thisItem) {
    if (dlgStack.Control("chkGroup").value == true) {
        if (Script.Config["extended_viewer_info"] == true) {
            if (thisItem.is_dir) { 
                var i = dlgStack.Control("listStack").AddItem(thisItem.name, 0, 1);
                dlgStack.Control("listStack").GetItemAt(i).subitems(0) = "";
            }
            else {
                var i = dlgStack.Control("listStack").AddItem(thisItem.name, 0, 2);
                if (DOpus.FSUtil.Exists(thisItem)) {
                    dlgStack.Control("listStack").GetItemAt(i).subitems(0) = thisItem.size.fmt;
                }
                else {
                    dlgStack.Control("listStack").GetItemAt(i).subitems(0) = "...";
                }
            }
            if (DOpus.FSUtil.Exists(thisItem)) {
                dlgStack.Control("listStack").GetItemAt(i).subitems(1) = thisItem.modify;
                dlgStack.Control("listStack").GetItemAt(i).subitems(2) = thisItem.attr_text;
                dlgStack.Control("listStack").GetItemAt(i).subitems(3) = thisItem.path;
            }
            else {
                dlgStack.Control("listStack").GetItemAt(i).subitems(1) = "...";
                dlgStack.Control("listStack").GetItemAt(i).subitems(2) = "...";
                dlgStack.Control("listStack").GetItemAt(i).subitems(3) = "File no longer exists";
            }
        }
        if (Script.Config["extended_viewer_info"] == false) {
            if (thisItem.is_dir) { 
                var i = dlgStack.Control("listStack").AddItem(thisItem.realpath, 0, 1);
            }
            else {
                var i = dlgStack.Control("listStack").AddItem(thisItem.realpath, 0, 2);
            }
        }
    }

    if (dlgStack.Control("chkGroup").value == false) {
        if (Script.Config["extended_viewer_info"] == true) {
            if (thisItem.is_dir) { 
                var i = dlgStack.Control("listStack").AddItem(thisItem.name);
                dlgStack.Control("listStack").GetItemAt(i).subitems(0) = "";
            }
            else {
                var i = dlgStack.Control("listStack").AddItem(thisItem.name);
                if (DOpus.FSUtil.Exists(thisItem)) {
                    dlgStack.Control("listStack").GetItemAt(i).subitems(0) = thisItem.size.fmt;
                }
                else {
                    dlgStack.Control("listStack").GetItemAt(i).subitems(0) = "...";
                }
            }
            if (DOpus.FSUtil.Exists(thisItem)) {
                dlgStack.Control("listStack").GetItemAt(i).subitems(1) = thisItem.modify;
                dlgStack.Control("listStack").GetItemAt(i).subitems(2) = thisItem.attr_text;
                dlgStack.Control("listStack").GetItemAt(i).subitems(3) = thisItem.path;
            }
            else {
                dlgStack.Control("listStack").GetItemAt(i).subitems(1) = "...";
                dlgStack.Control("listStack").GetItemAt(i).subitems(2) = "...";
                dlgStack.Control("listStack").GetItemAt(i).subitems(3) = "File no longer exists";
            }
        }
        if (Script.Config["extended_viewer_info"] == false) {
            var i = dlgStack.Control("listStack").AddItem(thisItem.realpath);
        }
    }

    dlgStack.Control("listStack").GetItemAt(i).icon(thisItem.realpath, 0);
    
    if (thisItem.is_dir) {
        dlgStack.Control("listStack").GetItemAt(i).fg = FOLDERS_FG_COLOUR;
        dlgStack.Control("listStack").GetItemAt(i).bg = FOLDERS_BG_COLOUR;
    }
    else {
        dlgStack.Control("listStack").GetItemAt(i).fg = FILES_FG_COLOUR;
        dlgStack.Control("listStack").GetItemAt(i).bg = FILES_BG_COLOUR;
    }
}

function populateListIcon(dlgStack, thisItem, map) {

    if (dlgStack.Control("chkGroup").value == true) {
        if (thisItem.is_dir) { 
            var i = dlgStack.Control("listStack").AddItem(thisItem.name, 0, 1);
        }
        else {
            var i = dlgStack.Control("listStack").AddItem(thisItem.name, 0, 2);
        }
    }
    if (dlgStack.Control("chkGroup").value == false) {
        var i = dlgStack.Control("listStack").AddItem(thisItem.name);
    }

    dlgStack.Control("listStack").GetItemAt(i).icon(thisItem.realpath, 0);

    if (thisItem.is_dir) {
        dlgStack.Control("listStack").GetItemAt(i).fg = FOLDERS_FG_COLOUR;
        dlgStack.Control("listStack").GetItemAt(i).bg = FOLDERS_BG_COLOUR;
    }
    else {
        dlgStack.Control("listStack").GetItemAt(i).fg = FILES_FG_COLOUR;
        dlgStack.Control("listStack").GetItemAt(i).bg = FILES_BG_COLOUR;
    }

    map(thisItem.name) = thisItem.realpath;
}

function viewPopupMenu(dlg, dlgStack, vecStack, scriptCmdData, map) {
    var dlgViewMenu = DOpus.Dlg;
    dlgViewMenu.choices = DOpus.Create.Vector("Open",                           // 1
                                                "Show",                         // 2
                                                "Go to...",                     // 3
                                                "-",                            // 4
                                                "Copy",                         // 5
                                                "Copy File Name",               // 6
                                                "Copy Full Pathname",           // 7
                                                "-",                            // 8
                                                "Properties",                   // 9
                                                "-",                            // 10
                                                "Remove from Stack",            // 11
                                                "-",                            // 12
                                                "Clear Filter");                // 13
    if (dlgStack.Control("editFilter").value == "") {
        dlgViewMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8);
    }
    else {
        dlgViewMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    }
    var viewMenuReturn = dlgViewMenu.Show;
    var cmd = scriptCmdData.func.command;
	switch (viewMenuReturn) {
        case 1: // Open
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                cmd.RunCommand("FileType ACTION=dblclk FILE \"" + strFullPath + "\"");
            }
            break;
        case 2: // Show
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                cmd.RunCommand("Show \"" + strFullPath + "\"");
            }
            break;
        case 3: // Go to...
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                cmd.RunCommand("Go \"" + strFullPath + "\"");
            }
            break;
        case 5: // Copy
            var vecFile = DOpus.NewVector();
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                var thisItem = DOpus.FSUtil.GetItem(strFullPath);
                vecFile.push_back = thisItem.realpath;
                DOpus.SetClip(vecFile);
            }
            break;
        case 6: // Copy File Name
            if (Script.Config["extended_viewer_info"]) {
                var strName = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
            }
            else {
                var item = DOpus.FSUtil.GetItem(dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name);
                var strName = item.name;
            }
            DOpus.SetClip(strName);
            break;
        case 7: // Copy Full Pathname
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                DOpus.SetClip(strFullPath);
            }
            break;
        case 9: // Properties
            if (dlgStack.Control("listStack").mode == "details") {
                if (Script.Config["extended_viewer_info"]) {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
                else {
                   var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                }
            }
            else {
                for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                    var key = eItem.item();
                    if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                        strFullPath = map(key);
                    }
                }
            }
            if (DOpus.FSUtil.Exists(strFullPath)) {
                cmd.RunCommand("Properties FILE \"" + strFullPath + "\"");
            }
            break;
        case 11: // Remove from Stack
            var vecViewTemp = DOpus.NewVector();
            for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                if (dlgStack.Control("listStack").mode == "details") {
                    if (Script.Config["extended_viewer_info"]) {
                       var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).subitems(3) + "\\" + dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                    }
                    else {
                       var strFullPath = dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name;
                    }
                }
                else {
                    for (var eItem = new Enumerator(map); !eItem.atEnd(); eItem.moveNext()) {
                        var key = eItem.item();
                        if (key == dlgStack.Control("listStack").GetItemAt(dlgStack.Control("listStack").value).name) {
                            strFullPath = map(key);
                        }
                    }
                }
                
                var strThisItem = String(thisItem.realpath);
                if (strThisItem != strFullPath) {
                    vecViewTemp.push_back = thisItem.realpath;
                }
            }
            vecStack.assign(vecViewTemp);
            updateViewer(dlg, dlgStack, vecStack, map);
            DOpus.SetClip(vecStack);
            updateDialog(dlg, vecStack);
            break;
        case 13: // Clear Filter
            dlgStack.Control("editFilter").value = "";
    }
}

function stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop) {
    var factory = DOpus.Create();
    var strSet = factory.StringSet();

    for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
        var stackItem = DOpus.FSUtil.GetItem(eItems.item());
        if (!strSet.insert(stackItem.realpath)) {
            vecLastDrop.clear();
        }
    }
    vecStack.clear();
    for (var eItems = new Enumerator(strSet); !eItems.atEnd(); eItems.moveNext()) {
        var strItem = DOpus.FSUtil.GetItem(eItems.item());
        vecStack.push_back = strItem.realpath;    
    }

    if (vecBeforeDrop == vecStack) {
        LAST_DROP_EXISTS = false;
    }
}

function updateDialog(dlg, vecStack) {
    dlg.Control("staticName").visible = false;
    dlg.Control("editName").visible = false;
    dlg.Control("btnNo").visible = false;
    dlg.Control("staticName").enabled = false;
    dlg.Control("editName").enabled = false;
    dlg.Control("btnNo").enabled = false;
    var intTotalFolders = 0;
    var intTotalFiles = 0;
    if (vecStack.empty == false) {
        for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
            var stackItem = DOpus.FSUtil.GetItem(eItems.item());
            if (stackItem.is_dir) {
                intTotalFolders = intTotalFolders + 1;
            }
            else{
                intTotalFiles = intTotalFiles + 1;
            }
        }
    }    
    if (intTotalFiles < 1 && intTotalFolders  < 1) {
        dlg.Control("staticStats").label = "Empty";
        dlg.Control("staticDrop").label = IMG_TARGET;
    }
    else {
        dlg.Control("staticDrop").label = IMG_STACK;
        dlg.Control("staticStats").label = intTotalFiles + " Files, " + intTotalFolders + " Folders";
    }                
}

function popupMenu (dlg, vecStack, vecLastDrop, vecBeforeDrop, scriptCmdData, srcTab) {
    
    var dlgMenu = DOpus.Dlg;
    dlgMenu.choices = DOpus.Create.Vector("Add Selected to Stack",                  // 1
                                            "Add Source to Stack",                  // 2
                                            "-",                                    // 3
                                            "Remove Last",                          // 4
                                            "Remove All",                           // 5
                                            "-",                                    // 6
                                            "Save Stack...",                        // 7
                                            "Load Stack...",                        // 8
                                            "-",                                    // 9
                                            "Copy",                                 // 10
                                            "Paste",                                // 11
                                            "-",                                    // 12
                                            "Copy File Names",                      // 13
                                            "Copy Full Pathnames",                  // 14
                                            "-",                                    // 15
                                            "Copy to Collection",                   // 16
                                            "Empty Collection",                     // 17
                                            "-",                                    // 18
                                            "Compress to archive...",               // 19
                                            "Compress and email...",                // 20
                                            "-",                                    // 21
                                            "Stack Viewer...",                      // 22
                                            "Change Stack Label",                   // 23
                                            "-",                                    // 24
                                            "About DopStack...",                    // 25
                                            "Exit DopStack");                       // 26
    
    if (vecStack.empty == false) { // Stack has items
        dlgMenu.menu = DOpus.Create.Vector(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
        if (LAST_DROP_EXISTS == false) {
            dlgMenu.menu = DOpus.Create.Vector(0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
        }
    }
    else {
        dlgMenu.menu = DOpus.Create.Vector(0, 0, 0, 8, 8, 0, 8, 0, 0, 8, 0, 0, 8, 8, 0, 8, 0, 0, 8, 8, 0, 8, 0, 0, 0, 0);
    }
    var menuReturn = dlgMenu.Show;

	switch (menuReturn) {
        case 1: // Add Selected to Stack
            dlg.KillTimer("timer");
            vecBeforeDrop.assign(vecStack);
            vecLastDrop.clear();
            DOpus.listers.Update();
            if (srcTab = DOpus.listers.lastactive.activetab) {
                srcTab.Update();
                for (var eItems = new Enumerator(srcTab.selected); !eItems.atEnd(); eItems.moveNext()) {
                    var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                    vecStack.push_back = thisItem.realpath;
                    vecLastDrop.push_back = thisItem.realpath;
                    DOpus.SetClip(vecStack);
                }
                LAST_DROP_EXISTS = true;
            }
            else {
                dlg.Control("staticStats").label = "No source";
                DOpus.Delay(1000);
            }
            stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
            updateDialog(dlg, vecStack);
            dlg.SetTimer(500, "timer");
            break;
        case 2: // Add source to Stack
            dlg.KillTimer("timer");
            vecBeforeDrop.assign(vecStack);
            vecLastDrop.clear();
            DOpus.listers.Update();
            if (srcTab = DOpus.listers.lastactive.activetab) {
                srcTab.Update();
                for (var eItems = new Enumerator(srcTab.all); !eItems.atEnd(); eItems.moveNext()) {
                    var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                    vecStack.push_back = thisItem.realpath;
                    vecLastDrop.push_back = thisItem.realpath;
                    DOpus.SetClip(vecStack);
                }
                LAST_DROP_EXISTS = true;
            }
            else {
                dlg.Control("staticStats").label = "No source";
                DOpus.Delay(1000);
            }
            stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
            updateDialog(dlg, vecStack);
            dlg.SetTimer(500, "timer");
            break;
        case 4: // Remove Last Added
            vecStack.assign(vecBeforeDrop);
            DOpus.SetClip(vecStack);
            updateDialog(dlg, vecStack);
            LAST_DROP_EXISTS = false;
            break;
        case 5: // Remove All
            vecStack.clear(); 
            updateDialog(dlg, vecStack);
            DOpus.SetClip();
            break;
        case 7: // Save Stack...
            saveStack(dlg, vecStack);
            break;
        case 8: // Load Stack...
            loadStack(dlg, vecStack, vecLastDrop, vecBeforeDrop);
            break;
        case 10: // Copy
            DOpus.SetClip(vecStack); 
            dlg.Control("staticStats").label = "Copied";
            DOpus.Delay(1000);
            updateDialog(dlg, vecStack);
            break;
        case 11: // Paste
            if (DOpus.GetClipFormat() == "files") {    
                vecBeforeDrop.assign(vecStack);
                vecLastDrop.clear();
                for (var eItems = new Enumerator(DOpus.GetClip("files")); !eItems.atEnd(); eItems.moveNext()) {
                    var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                    vecStack.push_back = thisItem.realpath;
                    vecLastDrop.push_back = thisItem.realpath;
                    DOpus.SetClip(vecStack);
                }
                LAST_DROP_EXISTS = true;
            }
            else {
                dlg.Control("staticStats").label = "No files in Clipboard";
                DOpus.Delay(1000);
            }
            stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
            updateDialog(dlg, vecStack);
            break;
        case 13: // Copy Filenames
            var strTextClip = "";
            for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                strTextClip = strTextClip + thisItem.name + "\\n";
            }   
            scriptCmdData.func.command.RunCommand("Clipboard EXPANDNEWLINES SET " + strTextClip); 
            break;
        case 14: // Copy Full Pathnames
            var strTextClip = "";
            for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                strTextClip = strTextClip + thisItem.realpath + "\\n";
            }   
            scriptCmdData.func.command.RunCommand("Clipboard EXPANDNEWLINES SET " + strTextClip); 
            break;
        case 16:  // Copy to Collection
            for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
                if (DOpus.FSUtil.Exists("coll://DopStack") == false) {
                    scriptCmdData.func.command.RunCommand("CreateFolder \"coll://DopStack\"");
                }
                    var thisItem = DOpus.FSUtil.GetItem(eItems.item());
                    scriptCmdData.func.command.RunCommand("Copy COPYTOCOLL=sub TO=\"coll://DopStack\" FILE=\"" + thisItem.realpath + "\"");
            }
            break;
        case 17: //Empty Collection
            scriptCmdData.func.command.RunCommand("Delete \"coll://DopStack\" QUIET NORECYCLE"); 
            scriptCmdData.func.command.RunCommand("CreateFolder \"coll://DopStack\"");
            dlg.Control("staticStats").label = "Emptied";
            DOpus.Delay(1000);
            updateDialog(dlg, vecStack);
            break;
        case 19: // Compress to archive
            switch (Script.Config["archive_format"]) {
                case 0: // .7z
                    var arcFile = "DopStack.7z";
                    break;
                case 1: // .zip
                    var arcFile = "DopStack.zip";
                    break;
                case 2: // .rar
                    var arcFile = "DopStack.rar";
            }
            var savePath = DOpus.FSUtil.Resolve(Script.Config["default_save_load_path"]);
            var arcItem = dlg.Save("Save As", savePath + "\\" + arcFile, "#", dlg);
            if (arcItem.result != false) {
                archiveCreate(dlg, srcTab, vecStack, scriptCmdData, arcItem);
            }
            break;
        case 20: // Compress and email
            NAME_USE = "email";
            dlg.Control("staticName").visible = true;
            dlg.Control("editName").visible = true;
            dlg.Control("btnNo").visible = true;
            dlg.Control("staticName").enabled = true;
            dlg.Control("staticName").label = "Enter archive name";
            dlg.Control("editName").enabled = true;
            dlg.Control("btnNo").enabled = true;
            dlg.Control("staticDrop").label = "";

            switch (Script.Config["archive_format"]) {
                case 0: // .7z
                    dlg.Control("editName").value = "DopStack.7z";
                    break;
                case 1: // .zip
                    dlg.Control("editName").value = "DopStack.zip";
                    break;
                case 2: // .rar
                    dlg.Control("editName").value = "DopStack.rar";
            }
            
            dlg.Control("editName").focus = true;
            dlg.Control("editName").SelectRange(0,-1);
            break;
        case 22: // Stack Viewer
            viewStack(dlg, vecStack, vecBeforeDrop, vecLastDrop, scriptCmdData);
            //updateDialog(dlg, vecStack);
            break;
        case 23: // Change Stack Label
            NAME_USE = "label";
            dlg.Control("editName").value = dlg.Control("staticLabel").label;
            dlg.Control("staticName").visible = true;
            dlg.Control("editName").visible = true;
            dlg.Control("btnNo").visible = true;
            dlg.Control("staticName").enabled = true;
            dlg.Control("staticName").label = "Enter stack name";
            dlg.Control("editName").enabled = true;
            dlg.Control("btnNo").enabled = true;
            dlg.Control("staticDrop").label = "";
            dlg.Control("editName").focus = true;
            dlg.Control("editName").SelectRange(0,-1);
            break;
        case 25: // About...
            OnAboutScript(null, dlg);
            break;
        case 26: // Exit
            exitConfirm(dlg, vecStack);
    }
}  

function saveStack(dlg, vecStack) {

    var saveName = "Default Stack.stack"
    
    var saveItem = dlg.Save("Save As", SAVE_LOAD_PATH + "\\" + saveName, "#", dlg);

    if (saveItem.result != false) {
        var fso = new ActiveXObject("Scripting.FilesystemObject");
        var stackText = fso.OpenTextFile(saveItem, 2, true);

        for (var eItems = new Enumerator(vecStack); !eItems.atEnd(); eItems.moveNext()) {
            var thisItem = DOpus.FSUtil.GetItem(eItems.item());
            stackText.WriteLine(thisItem.realpath);
        }
        stackText.Close();
    }
}

function loadStack(dlg, vecStack, vecLastDrop, vecBeforeDrop) {
    
    var loadItem = dlg.Open("Open File", SAVE_LOAD_PATH, dlg);
    var arrStack = [];
    vecBeforeDrop.assign(vecStack);
    vecLastDrop.clear();
    if (loadItem.result != false) {
        var fso = new ActiveXObject("Scripting.FilesystemObject");
        var stackFile = fso.OpenTextFile(loadItem, 1, false);

        if (fso.FileExists(loadItem)) {
            if (!stackFile.AtEndOfStream) {
                var stackContents = stackFile.ReadAll();
                arrStack = stackContents.split(/\r\n|\n/);
            }
        }
        stackFile.Close();
        for (i=0; i < arrStack.length; i ++) {
            if (DOpus.FSUtil.Exists(arrStack[i])) {
                vecStack.push_back = arrStack[i];
                vecLastDrop.push_back = arrStack[i];
            }
        }
        DOpus.SetClip(vecStack);
        checkExists(vecStack);
        stripDuplicates(vecStack, vecLastDrop, vecBeforeDrop);
        updateDialog(dlg, vecStack);
        LAST_DROP_EXISTS = true;
    }
}

function OnAboutScript(aboutData, dlg){
    dlgA = DOpus.Dlg;
    if (dlg) {
        dlgA.window = dlg;
        dlgA.disable_window = dlg;
    }
    else {
        dlgA.window = aboutData.window;
    }
    dlgA.title = scriptName + " - Directory Opus";
    dlgA.message = scriptName + " v" + scriptVersion + "\t\t\t\t" + scriptDate + "\n\n" + scriptDesc + "\n\n" + scriptCopyright;
    dlgA.buttons = "Forum...|Close";
    dlgA.icon = "info";
    var result = dlgA.show;

    result ? DOpus.Create.Command.RunCommand(scriptUrl) : null;
}

==SCRIPT RESOURCES
<resources>
	<resource name="dlgConfigErrors" type="dialog">
		<dialog fontsize="8" height="194" lang="english" resize="yes" standard_buttons="ok" title="DopStack Config Errors" width="144">
			<control fullrow="yes" height="168" name="listErrors" nosel="yes" nosortheader="yes" resize="wh" type="listview" viewmode="details" width="136" x="4" y="4" />
		</dialog>
	</resource>
	<resource name="dlgDopStack" type="dialog">
		<dialog dragdrop="yes" fontsize="8" height="89" lang="english" resize="yes" title="DopStack" width="92">
			<control enable="no" halign="center" height="8" name="staticError" notify="yes" title="!" type="static" visible="no" width="9" x="79" y="4" />
			<control halign="left" height="12" name="editName" resize="yw" type="edit" visible="no" width="84" x="4" y="38" />
			<control halign="left" height="8" name="staticName" resize="yw" title="Enter a name" type="static" visible="no" width="64" x="4" y="29" />
			<control height="9" name="btnNo" resize="xy" title="X" type="button" visible="no" width="10" x="77" y="28" />
			<control default="yes" height="14" name="btnEnter" type="button" visible="no" width="50" x="21" y="53" />
			<control halign="center" height="8" name="staticStats" notify="yes" resize="yw" title="test" type="static" width="92" x="0" y="71" />
			<control dragsource="yes" halign="left" height="71" image="yes" name="staticDrop" notify="yes" resize="wh" type="static" width="92" x="0" y="0" />
			<control halign="center" height="10" name="staticLabel" notify="yes" resize="yw" title="label" type="static" width="92" x="0" y="79" />
		</dialog>
	</resource>
	<resource name="dlgStackView" type="dialog">
		<dialog dragdrop="yes" fontsize="8" height="189" lang="english" resize="yes" width="262">
			<control halign="left" height="12" name="editFilter" resize="w" type="edit" width="98" x="30" y="5" />
			<control height="14" name="btnClear" resize="x" title="Clear" type="button" width="50" x="133" y="4" />
			<control dragsource="yes" fullrow="yes" height="145" name="listStack" resize="wh" smallicons="yes" type="listview" viewmode="details" width="252" x="5" y="22" />
			<control close="0" height="14" name="btnClose" resize="xy" title="Close" type="button" width="50" x="207" y="171" />
			<control halign="left" height="8" name="staticFilter" title="Filter:" type="static" width="21" x="5" y="7" />
			<control height="14" name="btnNotepad" resize="y" title="Notepad..." type="button" width="50" x="5" y="171" />
			<control height="10" name="chkGroup" resize="x" title="Group" type="check" width="32" x="227" y="6" />
			<control height="10" name="chkDetails" resize="x" title="Details" type="check" width="35" x="188" y="6" />
		</dialog>
	</resource>
</resources>
